ClrAllBuffers:
	ld hl,SaveSScreen
	call ClrBuffer
	ld hl,StatVars
	call ClrBuffer
	ld hl,AppBackUpScreen
	call ClrBuffer
	ld hl,PlotSScreen
	call ClrBuffer
	ret
ClrBackBuffers:
	ld hl,SaveSScreen
	call ClrBuffer
	ld hl,StatVars
	call ClrBuffer
	ret
ClrBuffer:
; Clears a screen buffer
;INPUT: hl = buffer
;Destroys: de
	ld d,h
	ld e,l
	inc de
	ld (hl),0
	ld bc,767
	ldir
	ret
MakeBufferBlack:
	ld d,h
	ld e,l
	inc de
	ld (hl),$FF
	ld bc,767
	ldir
	ret
	ret
XORScreen:
	ld bc,767
	ld hl,PlotSScreen
	ld de,AppBackUpScreen
	@:
		ld a,(hl)
		cpl
		ld (hl),a
		ld a,(de)
		cpl
		ld (de),a
		inc hl
		inc de
		dec bc
		xor a
		or b
		or c
		cp 0
		jr nz,{-1@}
	ret

;Display Tilemap
;Destroys: hl;de,bc,ix,ae
TileMap:
	ld hl,TileMapData+98 ; start at end and header offset
	
	ld d,0
	ld b,8
	ld c,0
	ld e,0
	jr _TileMapLoopRow
_TileMapLoopRow:
	push bc
	
	ld c,b
	ld b,12 ; 12 columns
	call _TileMapLoopCol
	
	pop bc
	djnz _TileMapLoopRow
	ret
_TileMapLoopCol:
	push bc
	
	dec hl; counter in map
	
	ld a,(hl)
	push hl
	ld h,0 ; a into hl
	ld l,a
	add hl,hl ; hl * 16
	add hl,hl
	add hl,hl
	add hl,hl
	
	
	
	ld de,sprites
	add hl,de
	
	push hl ; ld ix,hl
	pop ix
	
	push de
	
	ld a,c ;y pos
	dec a
	add a,a
	add a,a
	add a,a
	ld h,a
	
	ld a,b ;x pos
	dec a
	add a,a
	add a,a
	add a,a
	
	ld b,8
	
	
	call GraySpriteXOR
	
	
	pop de
	
	
	pop hl
	
	pop bc
	djnz _TileMapLoopCol
	ret


; updates screen
; takes:none
; destroys:de,hl,bc
Update:
	ld hl,SaveSScreen
	ld de,PlotSScreen
	ld bc,768
	ldir
	ld hl,StatVars
	ld de,AppBackUpScreen
	ld bc,768
	ldir
	ret

GraySpriteXOR:
	push af
	push hl
	push bc
	ld de,SaveSScreen
	call PutSpriteXOR
	;no need to 'save' ix as sprite routine offsets it correctly
	pop bc
	pop hl
	pop af
	ld de,StatVars
	call PutSpriteXOR
	ret
;sprite XOR
;a - xpos
;h - ypos
;b - height
;ix - sprite pointer
;de - buffer
;Destroys: hl,de,c
PutSpriteXOR:
	push de ; save the buffer
		ld e,h
		ld h,0
		ld l,e
		ld d,0
		ld e,l
		add	 hl,hl ;hl times 12
		add	 hl,de
		add	 hl,hl
		add	 hl,hl
		
		ld  e,a ;devide by 8
		srl e
		srl e
		srl e
		
		add hl,de ;total offset now in hl
		
		
	pop de ; restore the buffer
	add hl,de ; offset now with buffer
	
	and 7 ;is sprite aligned to 8x8 grid?
	
	jr z,_PutSpriteXOR_aligned
	ld c,a ; load the offset
	
	
_PutSpriteXOR_not_aligned:
	ld a,(ix) ; fetch byte
	
	push bc
	
	ld b,c ; left half
	call ShiftR ; shift a by offset
	xor (hl)
	ld (hl),a
	
	inc hl
	
	ld a,8 ; right half
	sub c
	ld b,a
	ld a,(ix)
	call ShiftL
	xor (hl)
	ld (hl),a
	
	pop bc
	
	inc ix
	
	ld de,11
	add hl,de
	
	djnz _PutSpriteXOR_not_aligned
	ret
	
_PutSpriteXOR_aligned:
	ld a,(ix) ; fetch byte
	xor (hl) ; xor
	ld (hl),a ; put it on screen
	inc ix ; on to the next byte!

	ld de,12 ; add column
	add hl,de

	djnz _PutSpriteXOR_aligned
	ret

;sprite AND
;a - xpos
;h - ypos
;b - height
;ix - sprite pointer
;de - buffer
;Destroys: hl,de,c
PutSpriteAND:
	push de ; save the buffer
		ld e,h
		ld h,0
		ld l,e
		ld d,0
		ld e,l
		add	 hl,hl ;hl times 12
		add	 hl,de
		add	 hl,hl
		add	 hl,hl
		
		ld  e,a ;devide by 8
		srl e
		srl e
		srl e
		
		add hl,de ;total offset now in hl
		
		
	pop de ; restore the buffer
	add hl,de ; offset now with buffer
	
	and 7 ;is sprite aligned to 8x8 grid?
	
	jr z,_PutSpriteAND_aligned
	ld c,a ; load the offset
	
	
_PutSpriteAND_not_aligned:
	ld a,(ix) ; fetch byte
	
	push bc
	
	ld b,c ; left half
	call ShiftR_1 ; shift a by offset
	and (hl)
	ld (hl),a
	
	inc hl
	
	ld a,8 ; right half
	sub c
	ld b,a
	ld a,(ix)
	call ShiftL_1
	and (hl)
	ld (hl),a
	
	pop bc
	
	inc ix
	
	ld de,11
	add hl,de
	
	djnz _PutSpriteAND_not_aligned
	ret
	
_PutSpriteAND_aligned:
	ld a,(ix) ; fetch byte
	and (hl) ; and
	ld (hl),a ; put it on screen
	inc ix ; on to the next byte!

	ld de,12 ; add column
	add hl,de

	djnz _PutSpriteAND_aligned
	ret


;Shift b times right
;a - shift
;b - times
ShiftR:
	srl a
	djnz ShiftR
	ret

;Shift b times left
;a - shift
;b - times
ShiftL:
	sla a
	djnz ShiftL
	ret

;Shift b times right adding 1s
;a - shift
;b - times
ShiftR_1:
	xor $FF
@:
	srl a
	djnz {-1@}
	xor $FF
	ret

;Shift b times left adding 1s
;a - shift
;b - times
ShiftL_1:
	xor $FF
@:
	sla a
	djnz {-1@}
	xor $FF
	ret


GetSprite:
; gets sprite at (c,l), destiny pointer is ix, buffer pointer is de
; routine ripped from axe
_GetSpriteEntry:
	ld b,0
	ld h,0
	ld	a,l
	add	a,a
	add	a,l
	ld	l,a
	add	hl,hl
	add	hl,hl
	add	hl,de
	ld	a,c
	sra	c
	sra	c
	sra	c
	add	hl,bc
	and	%00000111
	ld	b,8
	jr	nz,___GetSpriteUnaligned
	ld	de,12
___GetSpriteAligned:
	ld	a,(hl)
	ld	(ix),a
	add	hl,de
	inc	ix
	djnz	___GetSpriteAligned
	ret
___GetSpriteUnaligned:
	ld	d,(hl)
	inc	hl
	ld	e,(hl)
	ex	de,hl
	ld	c,a
___GetSpriteShift:
	add	hl,hl
	dec	c
	jr	nz,___GetSpriteShift
	ld	(ix),h
	ld	hl,11
	add	hl,de
	inc	ix
	djnz	___GetSpriteUnaligned
	ret
__GetSpriteEnd: