
LoadInterrupt:
	push af
	in a,(6)
	push af
	  ld a,(basepage)
	  out (6),a
	  call Interrupt
	pop af
	out (6),a
	pop af
	ret

StartupProg:
	di ; turn off interrupts

	ld hl,StatVars+531
	ld de,textShadow
	ld bc,128-4
	ldir

	ld hl,StatVars+531+128-4
	ld de,5byteBackBuf
	ld bc,5
	ldir

	ld hl,StatVars+531+128-4+5
	ld de,progCurrent
	ld bc,108
	ldir



	; set up the jump handler
	ld hl,$8585
	ld (hl),$c3
	ld de,keyToStr ; that is where it will be
	inc hl
	ld (hl),e
	inc hl
	ld (hl),d

	;copy the interrupt loader
	ld hl,LoadInterrupt
	ld de,keyToStr
	ld bc,17
	ldir

	ld hl,interruptMask
	ld (hl),%00001000


	in a,(2) ; test for crystal timer support
	bit 7,a
	jr nz,{+1@}
		; we need to fall back
		set 1,(hl)
	@:
	ld a,(hl) ; activate interrupts
	out (3),a

	ret

StopProg:

	; make sure we are in the correct screen mode
	ld    a,$05
	out   ($10),a		     ;73cc into


	ld a,%00001011 ; disable custom interrupt stuff
	out (3),a


	; restore os contrast
	ld a,(contrast)
	add a,$D8
	out ($10),a


	ld hl,textShadow
	ld de,StatVars+531
	ld bc,128-4
	ldir

	ld hl,5byteBackBuf
	ld de,StatVars+531+128-4
	ld bc,5
	ldir

	ld hl,progCurrent
	ld de,StatVars+531+128-4+5
	ld bc,108
	ldir

	ld hl,listEditPtrs
	ld (hl),0
	ld de,listEditPtrs+1
	ld bc,121+48-1
	ldir


	bcall(_DelRes) ; make sure we don't have the coordinates bug anymore
	bcall(_clrTxtShd)
	bcall(_fillBasePageTable)

	ld iy,flags ; restore iy
	ret

GrayTuner:
	call __ClrBackBuffers
	
	
	ld hl,_GrayTuner_Row1
	ld de,backBuffer1 + 6*12
	ld bc,12
	ldir
	
	ld hl,_GrayTuner_Row2
	ld de,backBuffer2 + 6*12
	ld bc,12
	ldir
	
	ld hl,backBuffer1 + 6*12
	ld de,backBuffer1 + 7*12
	ld bc,768 - (7*12)
	ldir
	
	ld hl,backBuffer2 + 6*12
	ld de,backBuffer2 + 7*12
	ld bc,768 - (7*12)
	ldir
	
	call __Update
	ld bc,0
	ld a,1
	ld hl,graytuner_text
	bcall(_Text)
	
	ld iy,flags
	ld hl,(GSCounter)
	ld (tmp1),hl
_GrayTuner_Keyloop:
	call __Wait_Key_Release
	call __Wait_Key_Input
	
	bit 0,a ; down key, decrease contrast
	jr nz,{+1@}
	push af
		ld a,(GSContrast)
		dec a
		and %00111111
		ld (GSContrast),a
		or %11000000
		out ($10),a
	pop af
	@:
	bit 3,a ; up key, increase contrast
	jr nz,{+1@}
	push af
		ld a,(GSContrast)
		inc a
		and %00111111
		ld (GSContrast),a
		or %11000000
		out ($10),a
	pop af
	@:
	
	bit 2,a ; right key, decrease delay
	jr nz,{+1@}
	push af
		ld hl,GSCounter
		dec (hl)
		
		jr nz,$+3
		inc (hl)
		
		call __DisableInterrupts
		call __RestoreInterrupts
	pop af
	@:
	bit 1,a ; left key, increase delay
	jr nz,{+1@}
	push af
		ld hl,GSCounter
		inc (hl)
		
		jr nz,$+3
		dec (hl)
		
		call __DisableInterrupts
		call __RestoreInterrupts
	pop af
	@:
	
	bit 7,a
	jr z,_GrayTuner_Exit_nosave
	
	cpl
	and %01100000
	jr z,_GrayTuner_Keyloop
	; time to save it!
	call __DisableInterrupts
	
	ld hl,GSAppvarName ; we know that the appvar is in flash, so let's unarchie it!
	rst rMov9ToOP1
	bcall(_Arc_Unarc)
	inc de ; header
	inc de
	ld hl,GSCounter
	ld bc,2
	ldir
	
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_Arc_Unarc)
	call __RestoreScreenSettings

	call __RestoreInterrupts
	ld a,%10111110 ; listen to key groups
	out (1),a
	jp __Wait_Key_Release
_GrayTuner_Exit_nosave:
	ld hl,(tmp1)
	ld (GSCounter),hl
	
	call __DisableInterrupts
	
	ld a,(GSContrast)
	or %11000000
	out ($10),a
	
	call __RestoreInterrupts
	
	jp __Wait_Key_Release
_GrayTuner_Row1:
	.db $00,$00,$00,$00,$00,$00,$FF,$FF,$FF,$FF,$FF,$FF
_GrayTuner_Row2:
	.db $00,$00,$00,$FF,$FF,$FF,$00,$00,$00,$FF,$FF,$FF

GSAppvarName:
	.db AppVarObj,"GS",0
GSAppvarNameLen = $ - GSAppvarName - 2

loadInterruptSettings:

	call __DisableInterrupts
	
	xor a ; we'll keep track here if we need to prompt with the settings
	ld (tmp1),a
	
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_ChkFindSym)
	jr c,_loadInterruptSettings_No_AppVar
	
	ld a,b
	or a
	jr nz,_loadInterruptSettings_AppVar_in_Flash
	jr _loadInterruptSettings_AppVar_in_RAM
_loadInterruptSettings_No_AppVar:
	ld a,1 ; ok, we need to display the settings
	ld (tmp1),a
	ld hl,GSAppvarName ; create the appvar
	rst rMov9ToOP1
	ld hl,23
	bcall(_CreateAppVar)
	inc de ; header
	inc de
	ex de,hl
	
	in a,(2)
	bit 7,a
	jr z,{+1@}
		ld (hl),184
		jr {+2@}
	@:
		ld (hl),4
	@:
	inc hl
	
	ld a,(contrast) ; fetch os contrast as default
	add a,24
	ld (hl),a
	
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_ChkFindSym)
	
_loadInterruptSettings_AppVar_in_RAM:
	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_Arc_Unarc)
	bcall(_ChkFindSym)
_loadInterruptSettings_AppVar_in_Flash:

	ld hl,GSAppvarName
	rst rMov9ToOP1
	bcall(_ChkFindSym)
	ex de,hl
	
	ld a,1+2+7+GSAppvarNameLen+2
	
	@:
		bcall(_LoadDEIndPaged)
		dec a
		jr nz,{-1@}
	ld a,b
	
	ld bc,2
	ld de,GSCounter
	bcall(_FlashToRAM)
	ld a,(GSContrast)
	or %11000000
	out ($10),a
	call __RestoreInterrupts
	
	ld a,%10111110 ; listen to key groups
	out (1),a
	
	ld a,(tmp1)
	or a
	ret z
	jp GrayTuner




;; duplicate methods because of paging issues

__DisableInterrupts:

	xor a
	out ($30),a

	di

	im 1
	ret

__RestoreInterrupts:
	ld hl, $8000
	ld de, $8001
	ld (hl), $85
	ld bc, 257
	ldir


	in a,(2)
	bit 7,a
	jr z,{+1@}


		ld a,$40
		out ($30),a
		
		ld a,3
		out ($31),a
		
		ld a,(GSCounter)
		out ($32),a
		
		ld a,1
		out ($20),a
		jr {+2@}
	@:
		ld a,(GSCounter)
		ld (interruptCounter),a
		xor a
		out (4),a
	@:
	im 2
	ei
	ret


__Wait_Key_Input:
	in a,(1) ; get keypress
	cp $FF
	jr nc,__Wait_Key_Input
	ret
__Wait_Key_Release:
	push af
	@:
		in a,(1) ; get keypress
		cp $FF
		jr c,{-1@}
	pop af
	ret

__ClrBackBuffers:
	ld hl,SaveSScreen
	call __ClrBuffer
	ld hl,StatVars
	call __ClrBuffer
	ret

__ClrBuffer:
	ld d,h
	ld e,l
	inc de
	ld (hl),0
	ld bc,767
	ldir
	ret

__Update:
	ld hl,SaveSScreen
	ld de,PlotSScreen
	ld bc,768
	ldir
	ld hl,StatVars
	ld de,AppBackUpScreen
	ld bc,768
	ldir
	ret

__RestoreScreenSettings:
	@:
		in a,($10)
		bit 7,a
		jr nz,{-1@}
	ld    a,$07
	out   ($10),a		     ; set LCD to increase mode
	@:
		in a,($10)
		bit 7,a
		jr nz,{-1@}
	ld a,$01
	out ($10),a ; set to 8 bit mode
	ret
